// coverage:ignore-file
import 'dart:convert';

import 'package:film_gyaan/core.dart';

/// @nodoc
class CreditDetails extends Result {
  CreditDetails({
    required this.creditType,
    required this.department,
    required this.job,
    required this.media,
    required this.mediaType,
    required this.id,
    required this.person,
  });

  factory CreditDetails.fromJson(String str) => CreditDetails.fromMap(
        json.decode(str) as Map<String, dynamic>,
      );

  factory CreditDetails.fromMap(Map<String, dynamic> json) => CreditDetails(
        creditType: json['credit_type'] as String? ?? '',
        department: json['department'] as String? ?? '',
        job: json['job'] as String? ?? '',
        media: Media.fromMap(
          json['media'] as Map<String, dynamic>? ?? <String, dynamic>{},
        ),
        mediaType: json['media_type'] as String? ?? '',
        id: json['id'] as String? ?? '',
        person: PeopleDetails.fromMap(
          json['person'] as Map<String, dynamic>? ?? <String, dynamic>{},
        ),
      );

  String creditType;
  String department;
  String job;
  Media media;
  String mediaType;
  String id;
  PeopleDetails person;

  String toJson() => json.encode(toMap());

  Map<String, dynamic> toMap() => <String, dynamic>{
        'credit_type': creditType,
        'department': department,
        'job': job,
        'media': media.toMap(),
        'media_type': mediaType,
        'id': id,
        'person': person.toMap(),
      };
}

class Media {
  Media({
    required this.posterPath,
    required this.name,
    required this.firstAirDate,
    required this.voteAverage,
    required this.overview,
    required this.voteCount,
    required this.backdropPath,
    required this.id,
    required this.genreIds,
    required this.originalName,
    required this.originCountry,
    required this.originalLanguage,
    required this.popularity,
    required this.seasons,
    required this.mediaType,
  });

  factory Media.fromJson(String str) => Media.fromMap(
        json.decode(str) as Map<String, dynamic>,
      );

  factory Media.fromMap(Map<String, dynamic> json) => Media(
        posterPath: json['poster_path'] as String? ?? '',
        name: json['name'] as String? ?? '',
        firstAirDate: DateTime.tryParse(
          json['first_air_date'] as String? ?? '',
        ),
        voteAverage: json['vote_average'] as num? ?? 0,
        overview: json['overview'] as String? ?? '',
        voteCount: json['vote_count'] as int? ?? 0,
        backdropPath: json['backdrop_path'] as String? ?? '',
        id: json['id'] as int? ?? 0,
        genreIds: List<int>.from(
          (json['genre_ids'] as List<dynamic>? ?? <dynamic>[]).map<dynamic>(
            (dynamic x) => x as int,
          ),
        ),
        originalName: json['original_name'] as String? ?? '',
        originCountry: List<String>.from(
          (json['origin_country'] as List<dynamic>? ?? <dynamic>[])
              .map<dynamic>(
            (dynamic x) => x as String,
          ),
        ),
        originalLanguage: json['original_language'] as String? ?? '',
        popularity: json['popularity'] as num? ?? 0,
        seasons: List<Season>.from(
          (json['seasons'] as List<dynamic>? ?? <dynamic>[]).map<dynamic>(
            (dynamic x) => Season.fromMap(
              x as Map<String, dynamic>,
            ),
          ),
        ),
        mediaType: json['media_type'] as String? ?? '',
      );

  String posterPath;
  String name;
  DateTime? firstAirDate;
  num voteAverage;
  String overview;
  int voteCount;
  String backdropPath;
  int id;
  List<int> genreIds;
  String originalName;
  List<String> originCountry;
  String originalLanguage;
  num popularity;
  List<Season> seasons;
  String mediaType;

  String toJson() => json.encode(toMap());

  Map<String, dynamic> toMap() => <String, dynamic>{
        'poster_path': posterPath,
        'name': name,
        'first_air_date': firstAirDate == null
            ? null
            : "${firstAirDate!.year.toString().padLeft(4, '0')}-${firstAirDate!.month.toString().padLeft(2, '0')}-${firstAirDate!.day.toString().padLeft(2, '0')}",
        'vote_average': voteAverage,
        'overview': overview,
        'vote_count': voteCount,
        'backdrop_path': backdropPath,
        'id': id,
        'genre_ids': List<dynamic>.from(
          genreIds.map<dynamic>(
            (x) => x,
          ),
        ),
        'original_name': originalName,
        'origin_country': List<dynamic>.from(
          originCountry.map<dynamic>(
            (x) => x,
          ),
        ),
        'original_language': originalLanguage,
        'popularity': popularity,
        'seasons': List<dynamic>.from(
          seasons.map<dynamic>(
            (x) => x.toMap(),
          ),
        ),
        'media_type': mediaType,
      };
}
