import 'package:film_gyaan/core.dart';

final _logger = injectLogger('film_gyaan.dx.keywords');

/// A mixin for Movies which will be handling all the request made related
/// to keywords
mixin KeywordsDx on Core {
  /// Get Keyword Details
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/keywords/get-keyword-details).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Keyword> getKeywordDetails({
    required int keywordId,
  }) async {
    Ensure(keywordId > 0).isTrue('Keyword Id should be > 0');

    var params = KeywordDetailsParams(
      id: keywordId,
    );

    _logger.info('Getting keyword details for $keywordId');

    return defaultFlow<KeywordDetailsParams, Keyword>(
      core: this,
      params: params,
      serializer: (dynamic json) => Keyword.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get Keyword Movies
  ///
  /// Get the movies that belong to a keyword.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/keywords/get-movies-by-keyword).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MoviesList> getKeywordMovies({
    required int keywordId,
  }) async {
    Ensure(keywordId > 0).isTrue('Keyword Id should be > 0');

    var params = KeywordMoviesParams(
      id: keywordId,
    );

    _logger.info('Getting movies for the keyword $keywordId');

    return defaultFlow<KeywordMoviesParams, MoviesList>(
      core: this,
      params: params,
      serializer: (dynamic json) => MoviesList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }
}
